@@ -18,11 +18,6 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with wkhtmltopdf.  If not, see <http://www.gnu.org/licenses/>.
 
-#ifdef __WKHTMLTOX_UNDEF_QT_DLL__
-#ifdef QT_DLL
-#undef QT_DLL
-#endif
-#endif
 
 /**
  * \file pdf.h
@@ -96,10 +91,10 @@
  *      are replaced in this string, see the wkhtmltopdf manual.
  * - \b header.center The text to print in the center part of the header.
  * - \b header.right The text to print in the right part of the header.
- * - \b header.line Should a line be printed under the header? Must be either "true" or "false.
- * - \b header.space The amount of space to put between the header and the content, e.g. "1.8". Be
- *      aware that if this is to large the header will be printed outside the pdf document. This
- *      can be correct with the margin.top setting.
+ * - \b header.line Whether a line should be printed under the header (either "true" or "false").
+ * - \b header.spacing The amount of space to put between the header and the content, e.g. "1.8". Be
+ *      aware that if this is too large the header will be printed outside the pdf document. This
+ *      can be corrected with the margin.top setting.
  * - \b header.htmlUrl Url for a HTML document to use for the header.
  *
  * \section pagePdfGlobal Pdf global settings
@@ -115,6 +110,7 @@
  * - \b copies How many copies should we print?. e.g. "2".
  * - \b collate Should the copies be collated? Must be either "true" or "false".
  * - \b outline Should a outline (table of content in the sidebar) be generated and put into the PDF? Must be either "true" or false".
+
  * - \b outlineDepth The maximal depth of the outline, e.g. "4".
  * - \b dumpOutline If not set to the empty string a XML representation of the outline is dumped to this file.
  * - \b out The path of the output file, if "-" output is sent to stdout, if empty the output is stored in a buffer.
@@ -124,7 +120,6 @@
  * - \b margin.bottom Size of the bottom margin, e.g. "2cm"
  * - \b margin.left Size of the left margin, e.g. "2cm"
  * - \b margin.right Size of the right margin, e.g. "2cm"
- * - \b outputFormat The format of the output document, must be ether "", "pdf" or "ps".
  * - \b imageDPI The maximal DPI to use for images in the pdf document.
  * - \b imageQuality The jpeg compression factor to use when producing the pdf document, e.g. "92".
  * - \b load.cookieJar Path of file used to load and store cookies.
@@ -270,16 +265,12 @@ CAPI(int) wkhtmltopdf_extended_qt() {
 
 /**
  * \brief Return the version of wkhtmltopdf
- * Example: 0.10.0_beta4. The string is utf8 encoded and is owned by wkhtmltopdf.
+ * Example: 0.12.1-development. The string is utf8 encoded and is owned by wkhtmltopdf.
  *
  * \return Qt version
  */
 CAPI(const char *) wkhtmltopdf_version() {
-	return STRINGIZE(MAJOR_VERSION)"."STRINGIZE(MINOR_VERSION)"."STRINGIZE(PATCH_VERSION)
-#ifdef BUILD
-		"_"STRINGIZE(BUILD)
-#endif
-		;
+	return STRINGIZE(FULL_VERSION);
 }
 
 /**
@@ -302,7 +293,7 @@ CAPI(int) wkhtmltopdf_init(int use_graphics) {
 		int aa = 1;
 
 		bool ug = true;
-#if defined(Q_WS_X11) || defined(Q_WS_MACX)
+#if defined(Q_OS_UNIX) || defined(Q_OS_MAC)
 #ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
 		ug = use_graphics;
 		if (!ug) QApplication::setGraphicsSystem("raster");
@@ -345,6 +336,16 @@ CAPI(wkhtmltopdf_global_settings *) wkhtmltopdf_create_global_settings() {
 	return reinterpret_cast<wkhtmltopdf_global_settings *>(new settings::PdfGlobal());
 }
 
+/**
+ * \brief Destroy a global settings  object
+ *
+ * Normally one would not need to call this since ownership of the
+ * settings object is transfarred to the converter.
+ */
+CAPI(void) wkhtmltopdf_destroy_global_settings(wkhtmltopdf_global_settings * obj) {
+	delete reinterpret_cast<settings::PdfGlobal *>(obj);
+}
+
 /**
  * \brief Alter a setting in a global settings object
  *
@@ -391,6 +392,17 @@ CAPI(wkhtmltopdf_object_settings *) wkhtmltopdf_create_object_settings() {
 	return reinterpret_cast<wkhtmltopdf_object_settings *>(new settings::PdfObject());
 }
 
+/**
+ * \brief Destroy a global settings  object
+ *
+ * Normally one would not need to call this since ownership of the
+ * settings object is transfarred to the converter.
+ */
+CAPI(void) wkhtmltopdf_destroy_object_settings(wkhtmltopdf_object_settings * obj) {
+	delete reinterpret_cast<settings::PdfObject *>(obj);
+}
+
+
 /**
  * \brief Alter a setting in a object settings object
  *
