@@ -18,11 +18,6 @@
 // You should have received a copy of the GNU Lesser General Public License
 // along with wkhtmltopdf.  If not, see <http://www.gnu.org/licenses/>.
 
-#ifdef __WKHTMLTOX_UNDEF_QT_DLL__
-#ifdef QT_DLL
-#undef QT_DLL
-#endif
-#endif
 
 #include "pdfconverter_p.hh"
 #include <QAuthenticator>
@@ -35,6 +30,7 @@
 #include <QWebFrame>
 #include <QWebPage>
 #include <QWebSettings>
+#include <QXmlQuery>
 #include <algorithm>
 #include <qapplication.h>
 #include <qfileinfo.h>
@@ -47,6 +43,9 @@
 using namespace wkhtmltopdf;
 using namespace wkhtmltopdf::settings;
 
+#define STRINGIZE_(x) #x
+#define STRINGIZE(x) STRINGIZE_(x)
+
 const qreal PdfConverter::millimeterToPointMultiplier = 2.83464567;
 
 DLL_LOCAL QMap<QWebPage *, PageObject *> PageObject::webPageToObject;
@@ -78,9 +77,9 @@ bool DLL_LOCAL looksLikeHtmlAndNotAUrl(QString str) {
 
 PdfConverterPrivate::PdfConverterPrivate(PdfGlobal & s, PdfConverter & o) :
 	settings(s), pageLoader(s.load),
-	out(o), printer(0), painter(0), webPrinter(0)
+	out(o), printer(0), painter(0)
 #ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
-    , measuringHFLoader(s.load), hfLoader(s.load), tocLoader1(s.load), tocLoader2(s.load)
+    , webPrinter(0), measuringHFLoader(s.load), hfLoader(s.load), tocLoader1(s.load), tocLoader2(s.load)
 	, tocLoader(&tocLoader1), tocLoaderOld(&tocLoader2)
     , outline(0), currentHeader(0), currentFooter(0)
 #endif
@@ -151,13 +150,15 @@ void PdfConverterPrivate::beginConvert() {
 		fail();
 		return;
 	}
-#endif
+#else
     bool headerHeightsCalcNeeded = false;
+#endif
 
 	for (QList<PageObject>::iterator i=objects.begin(); i != objects.end(); ++i) {
 		PageObject & o=*i;
 		settings::PdfObject & s = o.settings;
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
         if (!s.header.htmlUrl.isEmpty() ) {
             if (looksLikeHtmlAndNotAUrl(s.header.htmlUrl)) {
                 emit out.error("--header-html should be a URL and not a string containing HTML code.");
@@ -195,6 +196,7 @@ void PdfConverterPrivate::beginConvert() {
                 o.footerReserveHeight = settings.margin.bottom.first + s.footer.spacing;
             }
         }
+#endif
 
 		if (!s.isTableOfContent) {
 			o.loaderObject = pageLoader.addResource(s.page, s.load, &o.data);
@@ -207,6 +209,7 @@ void PdfConverterPrivate::beginConvert() {
 	emit out.phaseChanged();
 	loadProgress(0);
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
     if (headerHeightsCalcNeeded) {
         // preload header/footer to check their heights
         measuringHFLoader.load();
@@ -218,22 +221,26 @@ void PdfConverterPrivate::beginConvert() {
         if (settings.margin.bottom.first == -1) {
             settings.margin.bottom.first = 10;
         }
-        
+
         // set static header/footer reserve heights
         objects[0].headerReserveHeight = settings.margin.top.first;
         objects[0].footerReserveHeight = settings.margin.bottom.first;
 
         pageLoader.load();
     }
+#else
+    pageLoader.load();
+#endif
 }
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
 // calculates header/footer height
 // returns millimeters
 qreal PdfConverterPrivate::calculateHeaderHeight(PageObject & object, QWebPage & header) {
     typedef QPair<QWebElement, QString> p_t;
 
     TempFile   tempObj;
-    QString    tempFile = tempObj.create(settings.outputFormat == "ps"?".ps":".pdf");
+    QString    tempFile = tempObj.create(".pdf");
 
     QPainter * testPainter = new QPainter();
     QPrinter * testPrinter = createPrinter(tempFile);
@@ -252,16 +259,15 @@ qreal PdfConverterPrivate::calculateHeaderHeight(PageObject & object, QWebPage &
     return (height / PdfConverter::millimeterToPointMultiplier);
 }
 
+#endif
+
 QPrinter * PdfConverterPrivate::createPrinter(const QString & tempFile) {
     QPrinter * printer = new QPrinter(settings.resolution);
     if (settings.dpi != -1) printer->setResolution(settings.dpi);
     //Tell the printer object to print the file <out>
 
-    printer->setOutputFormat(
-        (settings.outputFormat == "ps" || (settings.outputFormat == "" && settings.out.endsWith(".ps", Qt::CaseInsensitive)))?
-        QPrinter::PostScriptFormat : QPrinter::PdfFormat
-        );
     printer->setOutputFileName(tempFile);
+    printer->setOutputFormat(QPrinter::PdfFormat);
 
     if ((settings.size.height.first != -1) && (settings.size.width.first != -1)) {
         printer->setPaperSize(QSizeF(settings.size.width.first,settings.size.height.first + 100), settings.size.height.second);
@@ -271,10 +277,12 @@ QPrinter * PdfConverterPrivate::createPrinter(const QString & tempFile) {
 
     printer->setOrientation(settings.orientation);
     printer->setColorMode(settings.colorMode);
+    printer->setCreator("wkhtmltopdf " STRINGIZE(FULL_VERSION));
 
     return printer;
 }
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
 void PdfConverterPrivate::preprocessPage(PageObject & obj) {
 	currentObject++;
 	if (obj.settings.isTableOfContent) {
@@ -288,7 +296,7 @@ void PdfConverterPrivate::preprocessPage(PageObject & obj) {
 	int tot = objects.size();
 	progressString = QString("Object ")+QString::number(currentObject)+QString(" of ")+QString::number(tot);
 	emit out.progressChanged((currentObject)*100 / tot);
-	
+
 	painter->save();
 
 	if (viewportSize.isValid() && ! viewportSize.isEmpty()) {
@@ -301,7 +309,7 @@ void PdfConverterPrivate::preprocessPage(PageObject & obj) {
 	QWebPrinter wp(obj.page->mainFrame(), printer, *painter);
 	obj.pageCount = obj.settings.pagesCount? wp.pageCount(): 0;
 	pageCount += obj.pageCount;
-	
+
 	if (obj.settings.includeInOutline)
 		outline->addWebPage(obj.page->mainFrame()->title(), wp, obj.page->mainFrame(),
 							obj.settings, obj.localLinks, obj.anchors);
@@ -309,7 +317,7 @@ void PdfConverterPrivate::preprocessPage(PageObject & obj) {
 		outline->addEmptyWebPage();
 	painter->restore();
 }
-
+#endif
 
 /*!
  * Prepares printing out the document to the pdf file
@@ -328,20 +336,17 @@ void PdfConverterPrivate::pagesLoaded(bool ok) {
 			 lout = "/dev/stdout";
 		 else
 #endif
-			 lout = tempOut.create(settings.outputFormat == "ps"?".ps":".pdf");
+			 lout = tempOut.create(".pdf");
 	}
 	if (settings.out.isEmpty())
-	  lout = tempOut.create(settings.outputFormat == "ps"?".ps":".pdf");
+	  lout = tempOut.create(".pdf");
 
 	printer = new QPrinter(settings.resolution);
 	if (settings.dpi != -1) printer->setResolution(settings.dpi);
 	//Tell the printer object to print the file <out>
 
-	printer->setOutputFormat(
-		(settings.outputFormat == "ps" || (settings.outputFormat == "" && settings.out.endsWith(".ps", Qt::CaseInsensitive)))?
-	    QPrinter::PostScriptFormat : QPrinter::PdfFormat
-		);
 	printer->setOutputFileName(lout);
+	printer->setOutputFormat(QPrinter::PdfFormat);
 
 	//We currently only support margins with the same unit
 	if (settings.margin.left.second != settings.margin.right.second ||
@@ -353,9 +358,15 @@ void PdfConverterPrivate::pagesLoaded(bool ok) {
 	}
 
     //Setup margins and papersize
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
     printer->setPageMargins(settings.margin.left.first, objects[0].headerReserveHeight,
                                 settings.margin.right.first, objects[0].footerReserveHeight,
                                 settings.margin.left.second);
+#else
+    printer->setPageMargins(settings.margin.left.first, settings.margin.top.first,
+                                settings.margin.right.first, settings.margin.bottom.first,
+                                settings.margin.left.second);
+#endif
 
 	if ((settings.size.height.first != -1) && (settings.size.width.first != -1)) {
 		printer->setPaperSize(QSizeF(settings.size.width.first,settings.size.height.first), settings.size.height.second);
@@ -365,6 +376,7 @@ void PdfConverterPrivate::pagesLoaded(bool ok) {
 
 	printer->setOrientation(settings.orientation);
 	printer->setColorMode(settings.colorMode);
+	printer->setCreator("wkhtmltopdf " STRINGIZE(FULL_VERSION));
 
 	if (!printer->isValid()) {
 		emit out.error("Unable to write to destination");
@@ -469,14 +481,37 @@ void PdfConverterPrivate::loadTocs() {
 			style = obj.tocStyleFile.create(".xsl");
 			StreamDumper styleDump(style);
 			dumpDefaultTOCStyleSheet(styleDump.stream, ps.toc);
-		} else
-			style = MultiPageLoader::guessUrlFromString(style).toString();
+		}
 
 		QString path = obj.tocFile.create(".xml");
 		StreamDumper sd(path);
-		outline->dump(sd.stream, style);
+		outline->dump(sd.stream);
 
-		obj.loaderObject = tocLoader->addResource(path, ps.load);
+		QFile styleFile(style);
+		if (!styleFile.open(QIODevice::ReadOnly)) {
+			emit out.error("Could not read the TOC XSL");
+			fail();
+		}
+
+		QFile xmlFile(path);
+		if (!xmlFile.open(QIODevice::ReadOnly)) {
+			emit out.error("Could not read the TOC XML");
+			fail();
+		}
+
+		QString htmlPath = obj.tocFile.create(".html");
+		QFile htmlFile(htmlPath);
+		if (!htmlFile.open(QIODevice::WriteOnly)) {
+			emit out.error("Could not open the TOC for writing");
+			fail();
+		}
+
+		QXmlQuery query(QXmlQuery::XSLT20);
+		query.setFocus(&xmlFile);
+		query.setQuery(&styleFile);
+		query.evaluateTo(&htmlFile);
+
+		obj.loaderObject = tocLoader->addResource(htmlPath, ps.load);
 		obj.page = &obj.loaderObject->page;
 		PageObject::webPageToObject[obj.page] = &obj;
 		updateWebSettings(obj.page->settings(), ps.web);
@@ -504,9 +539,11 @@ void PdfConverterPrivate::findLinks(QWebFrame * frame, QVector<QPair<QWebElement
 	if (!ulocal && !uexternal) return;
 	foreach (const QWebElement & elm, frame->findAllElements("a")) {
 		QString n=elm.attribute("name");
+		if (n.isEmpty()) n=elm.attribute("ns0:name");
 		if (n.startsWith("__WKANCHOR_")) anchors[n] = elm;
 
 		QString h=elm.attribute("href");
+		if (h.isEmpty()) h=elm.attribute("ns0:href");
 		if (h.startsWith("__WKANCHOR_")) {
 			local.push_back( qMakePair(elm, h) );
 		} else {
@@ -600,6 +637,7 @@ void PdfConverterPrivate::endPage(PageObject & object, bool hasHeaderFooter, int
 	//object.headers[objectPage];
 	if (currentHeader) {
 		QWebPage * header = currentHeader;
+		updateWebSettings(header->settings(), object.settings.web);
 		painter->save();
 		painter->resetTransform();
 		double spacing = s.header.spacing * printer->height() / printer->heightMM();
@@ -628,6 +666,7 @@ void PdfConverterPrivate::endPage(PageObject & object, bool hasHeaderFooter, int
 
 	if (currentFooter) {
 		QWebPage * footer=currentFooter;
+		updateWebSettings(footer->settings(), object.settings.web);
 		painter->save();
 		painter->resetTransform();
 		double spacing = s.footer.spacing * printer->height() / printer->heightMM();
@@ -656,7 +695,6 @@ void PdfConverterPrivate::endPage(PageObject & object, bool hasHeaderFooter, int
 	}
 
 }
-#endif
 
 void PdfConverterPrivate::handleTocPage(PageObject & obj) {
 	painter->save();
@@ -670,6 +708,7 @@ void PdfConverterPrivate::handleTocPage(PageObject & obj) {
 	tocChanged = outline->replaceWebPage(obj.number, obj.settings.toc.captionText, wp, obj.page->mainFrame(), obj.settings, obj.localLinks, obj.anchors) || tocChanged;
 	painter->restore();
 }
+#endif
 
 
 void PdfConverterPrivate::tocLoaded(bool ok) {
@@ -731,6 +770,7 @@ void PdfConverterPrivate::measuringHeadersLoaded(bool ok) {
         return;
     }
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
     for (int d=0; d < objects.size(); ++d) {
         PageObject & obj = objects[d];
         if (obj.measuringHeader) {
@@ -743,6 +783,7 @@ void PdfConverterPrivate::measuringHeadersLoaded(bool ok) {
             obj.footerReserveHeight = calculateHeaderHeight(obj, *obj.measuringFooter) + obj.settings.header.spacing;
         }
     }
+#endif
 
     pageLoader.load();
 }
@@ -758,6 +799,7 @@ void PdfConverterPrivate::headersLoaded(bool ok) {
 	printDocument();
 }
 
+#ifdef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
 
 void PdfConverterPrivate::spoolPage(int page) {
 	progressString = QString("Page ") + QString::number(actualPage) + QString(" of ") + QString::number(actualPages);
@@ -815,7 +857,7 @@ void PdfConverterPrivate::spoolTo(int page) {
 			spoolPage(objectPage);
 		if (ps.pagesCount) ++pageNumber;
 		++objectPage;
-		
+
 		//TODO free header and footer
 		currentHeader=NULL;
 		currentFooter=NULL;
@@ -823,10 +865,10 @@ void PdfConverterPrivate::spoolTo(int page) {
 }
 
 void PdfConverterPrivate::beginPrintObject(PageObject & obj) {
-	if (obj.number != 0) 
+	if (obj.number != 0)
 		endPrintObject(objects[obj.number-1]);
 	currentObject = obj.number;
-	
+
 	if (!obj.loaderObject || obj.loaderObject->skip) return;
 	const settings::PdfObject & ps = obj.settings;
 	pageHasHeaderFooter = ps.header.line || ps.footer.line ||
@@ -841,27 +883,27 @@ void PdfConverterPrivate::beginPrintObject(PageObject & obj) {
 		foreach (QWebElement elm, obj.page->mainFrame()->findAllElements("textarea"))
 			elm.setStyleProperty("color","white");
 	}
-	
+
 	//output
 	webPrinter = new QWebPrinter(obj.page->mainFrame(), printer, *painter);
 	QString l1=obj.page->mainFrame()->url().path().split("/").back()+"#";
 	QString l2=obj.page->mainFrame()->url().toString() + "#";
-	
+
 	outline->fillAnchors(obj.number, obj.anchors);
 
 	//Sort anchors and links by page
 	for (QHash<QString, QWebElement>::iterator i=obj.anchors.begin();
 		 i != obj.anchors.end(); ++i)
 		pageAnchors[webPrinter->elementLocation(i.value()).first][i.key()] = i.value();
-	
+
 	for (QVector< QPair<QWebElement,QString> >::iterator i=obj.localLinks.begin();
 		 i != obj.localLinks.end(); ++i)
 		pageLocalLinks[webPrinter->elementLocation(i->first).first].push_back(*i);
-	
+
 	for (QVector< QPair<QWebElement,QString> >::iterator i=obj.externalLinks.begin();
 		 i != obj.externalLinks.end(); ++i)
 		pageExternalLinks[webPrinter->elementLocation(i->first).first].push_back(*i);
-	
+
 	if (ps.produceForms) {
 		foreach (const QWebElement & elm, obj.page->mainFrame()->findAllElements("input"))
 			pageFormElements[webPrinter->elementLocation(elm).first].push_back(elm);
@@ -897,14 +939,15 @@ void PdfConverterPrivate::endPrintObject(PageObject & obj) {
 
 	if (webPrinter != 0) {
 		QWebPrinter *tmp = webPrinter;
-		webPrinter = 0; 
+		webPrinter = 0;
 		delete tmp;
 
 		painter->restore();
 	}
 
 }
-			
+
+#endif
 
 void PdfConverterPrivate::printDocument() {
 #ifndef __EXTENSIVE_WKHTMLTOPDF_QT_HACK__
@@ -924,7 +967,7 @@ void PdfConverterPrivate::printDocument() {
 
 	progressString = "Preparing";
 	emit out.progressChanged(0);
-	
+
 	for (int cc_=0; cc_ < cc; ++cc_) {
 		pageNumber=1;
 		for (int d=0; d < objects.size(); ++d) {
@@ -948,7 +991,7 @@ void PdfConverterPrivate::printDocument() {
 
 	if (!settings.dumpOutline.isEmpty()) {
 		StreamDumper sd(settings.dumpOutline);
-		outline->dump(sd.stream, "");
+		outline->dump(sd.stream);
 	}
 
  	painter->end();
